// ConfigurationFactory.cpp

#include "stdafx.h"
#pragma hdrstop

#include "ConfigurationFactory.h"
#include "Configuration.h"
#include "TestFactory.h"
#include "Platform.h"
#include "Util.h"
#include "Constants.h"

namespace WaveSoftware
{
	bool ReadMain(TIniFile* ini, Configuration* config);
   bool ReadPlatform(TIniFile* ini, Configuration* config);
   void ReadTest(TIniFile* ini, Platform* platform, const std::string& name);
   
   Configuration* ConfigurationFactory::Create(const std::string& filename)
   {
      Configuration* config = new Configuration;
      bool validconfig = false;

      std::string drive;
      std::string dir;
      std::string fname;
      std::string ext;

      Util::SplitPath(filename, drive, dir, fname, ext);

      config->SetName(fname);

      if(0 == dir.length())
         fname = CSTR_PERIOD + CSTR_BACKSLASH + filename;
      else
         fname = filename;

      TIniFile* ini = new TIniFile(fname.c_str());

      if(ReadMain(ini, config))
         if(ReadPlatform(ini, config))
            validconfig = true;

      delete ini;  ini = NULL;

      if(!(validconfig))
      {
         delete config;  config = NULL;
      }
      
      return(config);
   }


   bool ReadMain(TIniFile* ini, Configuration* config)
   {
      // [Main] looks like:
      //
      // Title=Killer App 2.2
      // Copyright=Copyright (C) Wave Software Limited.  All Rights Reserved.
      // DetailsRoot=html
      // AlwaysDetails=yes
      // RegistryKey=software\wavesoftware\killerapp\2.2\install

      AnsiString tmp;

      tmp = ini->ReadString(CSTR_MAIN.c_str(),
                            CSTR_TITLE.c_str(),
                            CSTR_EMPTY_STRING.c_str());
      config->SetTitle(tmp.c_str());

      tmp = ini->ReadString(CSTR_MAIN.c_str(),
                            CSTR_COPYRIGHT.c_str(),
                            CSTR_EMPTY_STRING.c_str());
      config->SetCopyright(tmp.c_str());

      tmp = ini->ReadString(CSTR_MAIN.c_str(),
                            CSTR_DETAILSROOT.c_str(),
                            CSTR_EMPTY_STRING.c_str());
      config->SetDetailsRoot(tmp.c_str());

      tmp = ini->ReadString(CSTR_MAIN.c_str(),
                            CSTR_REGISTRYKEY.c_str(),
                            CSTR_EMPTY_STRING.c_str());
      config->SetRegistryKey(tmp.c_str());

      return(true);
   }

   bool ReadPlatform(TIniFile* ini, Configuration* config)
   {
      std::string cp = Util::GetCurrentPlatform();

      TStringList* sections = new TStringList;

      ini->ReadSections(sections);

      // Is the current platform a section?  If not, what we're
      // running on is not supported.
      if(-1 == sections->IndexOf(cp.c_str()))
         return(false);

      delete sections;  sections = NULL;

      Platform* platform = new Platform(cp);
      config->SetPlatform(platform);
      
      TStringList* keys = new TStringList;

      ini->ReadSection(cp.c_str(), keys);
      
      for(int i = 0; i < keys->Count; i++)
      {
         AnsiString key = keys->Strings[i];
         AnsiString val = ini->ReadString(cp.c_str(),
         	key,
            CSTR_EMPTY_STRING.c_str());

         ReadTest(ini, platform, val.c_str());
      }

      delete keys;  keys = NULL;

      return(true);
   }

   void ReadTest(TIniFile* ini, Platform* platform, const std::string& name)
   {
      StringMap data;
      TStringList* keys = new TStringList;

      ini->ReadSection(name.c_str(), keys);

      for(int i = 0; i < keys->Count; i++)
      {
         AnsiString key = keys->Strings[i];
         AnsiString val = ini->ReadString(name.c_str(),
         key,
         CSTR_EMPTY_STRING.c_str());

         data.insert(StringMap::value_type(key.c_str(), val.c_str()));
      }

      delete keys;  keys = NULL;

      Test* test = TestFactory::Create(data);

      if(NULL != test)
         platform->AddTest(test);
   }

} // end of namespace



