// FixedFileInfo.cpp

#include "stdafx.h"
#pragma hdrstop

#include "FixedFileInfo.h"

namespace WaveSoftware
{

FixedFileInfo::FixedFileInfo()
{
   m_Available = false;
}

bool FixedFileInfo::IsAvailable()
{
   return(m_Available);
}

void FixedFileInfo::SetPathname(const std::string& pathname)
{
   TCHAR buffer[_MAX_PATH];

   _tcscpy(buffer, pathname.c_str());
   ::PathUnquoteSpaces(buffer);

   m_Pathname = buffer;

   ReadFixedFileInfo();
}

const Version& FixedFileInfo::GetFileVersion()
{
   return(m_FileVersion);
}

const Version& FixedFileInfo::GetProductVersion()
{
   return(m_ProductVersion);
}

void FixedFileInfo::ReadFixedFileInfo()
{
   DWORD visize = 0;
   DWORD tmp    = 0;

   m_Available = false;

   visize = ::GetFileVersionInfoSize((TCHAR*) m_Pathname.c_str(), &tmp);

   if(visize > 0)
   {
      BYTE* data = new BYTE[visize];

      if(::GetFileVersionInfo((TCHAR*) m_Pathname.c_str(),
                              tmp, /* ignored */
                              visize,
                              (LPVOID*) data))
      {
         VS_FIXEDFILEINFO* ffi = NULL;
         UINT l = 0;

         if(::VerQueryValue((LPVOID) data,
                            _T("\\"),
                            (LPVOID*) &ffi,
                            &l))
         {
            m_FileVersion.SetMajor(HIWORD(ffi->dwFileVersionMS));
            m_FileVersion.SetMinor(LOWORD(ffi->dwFileVersionMS));
            m_FileVersion.SetRelease(HIWORD(ffi->dwFileVersionLS));
            m_FileVersion.SetBuild(LOWORD(ffi->dwFileVersionLS));

            m_ProductVersion.SetMajor(HIWORD(ffi->dwProductVersionMS));
            m_ProductVersion.SetMinor(LOWORD(ffi->dwProductVersionMS));
            m_ProductVersion.SetRelease(HIWORD(ffi->dwProductVersionLS));
            m_ProductVersion.SetBuild(LOWORD(ffi->dwProductVersionLS));

            m_Available = true;
         }
      }

      delete [] data;  data = NULL;
   }
}

} // end of namespace
