// FreeDiskSpace.cpp

#include "stdafx.h"
#pragma hdrstop

#include "FreeDiskSpace.h"
#include "Util.h"
#include "Constants.h"

namespace WaveSoftware
{

// Windows 98SE, ME, NT4.SP4, W2K, and XP have a "Ex" function for
// determining the size of "big" disks.  So that ECHECKLITE can run on
// 95, 98, and NT before SP4, we have to pick the function out of
// KERNEL32.DLL ourselves and if it is not available fall back on
// using the regular GetDiskFreeSpace() function.
typedef BOOL (WINAPI *PFN_GETDISKFREESPACE)(LPCTSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER);

FreeDiskSpace::FreeDiskSpace() :
      Test()
{
}

FreeDiskSpace::~FreeDiskSpace()
{
}

bool FreeDiskSpace::Initialise(const StringMap& data)
{
   if(!(Test::Initialize(data)))
      return(false);

   if(!(Util::GetFromMap(data, CSTR_FREEDISKSPACE, m_FreeDiskSpace)))
      return(false);

   return(true);
}

bool FreeDiskSpace::Verify()
{
   SetStatus(TS_HIGH);
   SetStatusMsg(CSTR_INSUFFICIENT);
   
   HINSTANCE kernel = ::LoadLibrary(CSTR_KERNEL32_DLL.c_str());
   PFN_GETDISKFREESPACE pfn_getdiskfreespace = NULL;

   if(NULL != kernel)
      pfn_getdiskfreespace =
         (PFN_GETDISKFREESPACE) ::GetProcAddress(kernel,
         CSTR_GETDISKFREESPACEEX.c_str());

   // Examine each "local" drive until one is found that has enough
   // free space or all the drives have been examined.

   // drives is a bit mask:
   //
   // A = 0
   // B = 2
   // C = 4
   // ...

   DWORD drives = ::GetLogicalDrives();
   bool found = false;

   for(int i = 0; (i < 26) && !(found); i++)
   {
      if(drives & (1 << i))
      {
         std::string root = ((char) ('A' + i)) + CSTR_COLON + CSTR_BACKSLASH;

         switch(::GetDriveType(root.c_str()))
         {
         case 0:
         case 1:
         case DRIVE_REMOVABLE:
         case DRIVE_CDROM:
         case DRIVE_RAMDISK:
            // Skip.
            break;

         case DRIVE_FIXED:
            if(NULL == pfn_getdiskfreespace)
            {
               // No extended function available, use regular
               // GetDiskFreeSpace.
               DWORD spc = 0;
               DWORD bps = 0;
               DWORD nfc = 0;
               DWORD tnc = 0;

               if(::GetDiskFreeSpace(root.c_str(),
                                     &spc,
                                     &bps,
                                     &nfc,
                                     &tnc))
               {
                  if(((spc * bps * nfc) / (1000 * 1000)) >
                  	(UINT) m_FreeDiskSpace)
                     found = true;
               }
            }
            else
            {
               ULARGE_INTEGER fbc = { 0L };
               ULARGE_INTEGER tbs = { 0L };
               ULARGE_INTEGER tfb = { 0L };

               if((*pfn_getdiskfreespace)(root.c_str(),
                                          &fbc,
                                          &tbs,
                                          &tfb))
               {
                  if((tfb.QuadPart / (1000 * 1000)) > m_FreeDiskSpace)
                     found = true;
               }
            }

         default:
            break;
         }
      }
   }

   if(found)
   {
      SetStatus(TS_OK);
      SetStatusMsg(CSTR_SUFFICIENT);
   }
      

   return(found);
}

} // end of namespace
