// MDAC.cpp

#include "stdafx.h"
#pragma hdrstop

#include "MDAC.h"
#include "Version.h"
#include "Constants.h"
#include "Util.h"
#include "mdacver_tlb.h"

namespace WaveSoftware
{

MDAC::MDAC() :
      Test()
{
}

MDAC::~MDAC()
{
   Util::DestroyVersions(m_Versions);
}

bool MDAC::Initialize(const StringMap& data)
{
   if(!(Test::Initialize(data)))
      return(false);

   std::string tmp;

   if(!(Util::GetFromMap(data, CSTR_VERSIONS, tmp)))
      return(false);

   if(!(Util::GetFromString(tmp, m_Versions)))
      return(false);

   return(true);
}

bool MDAC::Verify()
{
   TCOMIVersion objMDACVer;
   HRESULT hr = CoVersion::Create(objMDACVer);

   if(!(SUCCEEDED(hr)))
      return(true);		// The test succeeded, we just didn't find MDAC.

   TVariant vmaj = objMDACVer.get_Major();
   TVariant vmin = objMDACVer.get_Minor();

   // Build in this context refers to the release portion of the
   // version identifier.  Qfe is Microsoft's MDAC term for what we
   // call "build".
   TVariant vrel = objMDACVer.get_Build();
   TVariant vbld = objMDACVer.get_Qfe();

   Version v((int) vmaj, (int) vmin, (int) vrel, (int) vbld);

   int cr = Util::CompareVersionWithVersions(v, m_Versions);

   SetStatus((Test::TEST_STATUS) cr);
   SetStatusMsg(v.ToString());

   return(true);
}

} // end of namespace
