// RegistryFile.cpp

#include "stdafx.h"
#pragma hdrstop

#include "RegistryFile.h"
#include "FixedFileInfo.h"
#include "Util.h"
#include "Constants.h"

namespace WaveSoftware
{
   HKEY g_RootKeys[] =
   {
      HKEY_CLASSES_ROOT,	// 0
      HKEY_CURRENT_USER,	// 1
      HKEY_LOCAL_MACHINE	// 2
   };

   RegistryFile::RegistryFile()
   {
   }

   RegistryFile::~RegistryFile()
   {
   }

   bool RegistryFile::Initialize(const StringMap& data)
   {
      if(!(Test::Initialize(data)))
         return(false);

      int rki = 0;
      if(!(Util::GetFromMap(data, CSTR_ROOTKEY, rki)))
         return(false);

      if((rki < 0) || (rki >= sizeof(g_RootKeys)))
         return(false);

      std::string key;
      if(!(Util::GetFromMap(data, CSTR_KEY, key)))
         return(false);

      std::string value;
      if(!(Util::GetFromMap(data, CSTR_VALUE, value)))
         return(false);

      std::string dta;
      if(!(Util::GetFromMap(data, CSTR_DATA, dta)))
         return(false);

      std::string search;
      if(!(Util::GetFromMap(data, CSTR_SEARCH, search)))
         return(false);

      std::string replace;
      if(!(Util::GetFromMap(data, CSTR_REPLACE, replace)))
         return(false);

      std::string tmp;
      if(!(Util::GetFromMap(data, CSTR_VERSIONS, tmp)))
         return(false);

      if(!(Util::GetFromString(tmp, m_Versions)))
         return(false);

      // Build a suitable pathname from the data supplied.  Pathnames
      // are either directly read from the registry, i.e. Key points
      // to a registry key and the full pathname is available in the
      // default registry value.
      //
      // If Key is just a key, we read Value.  If Value is only a
      // directory, we append Data.
      //
      // Finally, the pathname can be processed by doing a
      // SEARCH/REPLACE using the values of Search and Replace.
      HKEY rootkey = g_RootKeys[rki];

      TRegistry* reg = new TRegistry;

      reg->RootKey = rootkey;
      if(reg->OpenKey(key.c_str(), false))
      {
         AnsiString pathname;

         pathname = reg->ReadString(value.c_str());

         if(dta.length() > 0)
            pathname = pathname + CSTR_BACKSLASH.c_str() + dta.c_str();

         if(search.length() > 0)
         {
            TReplaceFlags rf;

            rf << rfReplaceAll << rfIgnoreCase;

            pathname = StringReplace(pathname,
                                     search.c_str(),
                                     replace.c_str(),
                                     rf);
         }

         m_Pathname = pathname.c_str();

         reg->CloseKey();
      }

      delete reg;  reg = NULL;

      return(true);
   }

   bool RegistryFile::Verify()
   {
      SetStatus(TS_HIGH);
      SetStatusMsg(CSTR_UNDETECTED);

      FixedFileInfo ffi;

      ffi.SetPathname(m_Pathname);

      const Version& v = ffi.GetFileVersion();

      int cr = Util::CompareVersionWithVersions(v, m_Versions);

      SetStatus((Test::TEST_STATUS) cr);
      SetStatusMsg(v.ToString());

      return(true);
   }

} // end of namespace
