// ServicePack.cpp

#include "stdafx.h"
#pragma hdrstop

#include "ServicePack.h"
#include "Util.h"
#include "Constants.h"
#include <sstream>

namespace WaveSoftware
{

   ServicePack::ServicePack()
   {
      m_MinMajor = 0;
      m_MinMinor = 0;
      m_RecMajor = 0;
      m_RecMinor = 0;
   }

   ServicePack::~ServicePack()
   {
   }

   bool ServicePack::Initialize(const StringMap& data)
   {
      if(!(Test::Initialize(data)))
         return(false);

      if(!(Util::GetFromMap(data, CSTR_MINMAJOR, m_MinMajor)))
         return(false);

      if(!(Util::GetFromMap(data, CSTR_MINMINOR, m_MinMinor)))
         return(false);

      if(!(Util::GetFromMap(data, CSTR_RECMAJOR, m_RecMajor)))
         return(false);

      if(!(Util::GetFromMap(data, CSTR_RECMINOR, m_RecMinor)))
         return(false);

      return(true);
   }

   bool ServicePack::Verify()
   {
      SetStatus(TS_HIGH);
      SetStatusMsg(CSTR_UNDETECTED);

      OSVERSIONINFO osi;
      BOOL r = false;
      long sp = 0L;

      ::ZeroMemory(&osi, sizeof(OSVERSIONINFO));

      osi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);

      r = ::GetVersionEx(&osi);

      if(r)
      {
         TCHAR* csd = osi.szCSDVersion;

         while((*csd) && !(_istdigit(*csd)))
            ++csd;

         sp = MAKELONG(0, _ttoi(csd));
      }

      long min = MAKELONG(m_MinMinor, m_MinMajor);
      long rec = MAKELONG(m_RecMinor, m_RecMajor);

      if(sp < min)
         SetStatus(TS_HIGH);
      else if((sp >= min) && (sp < rec))
         SetStatus(TS_MED);
      else
         SetStatus(TS_OK);

      std::stringstream ss;

      if((0 == HIWORD(sp)) && (0 == LOWORD(sp)))
         ss << CSTR_NONE;
      else
      {
         ss << HIWORD(sp);

         if(0 != LOWORD(sp))
            ss << CSTR_PERIOD << LOWORD(sp);
      }

      SetStatusMsg(ss.str());

      return(true);
   }

} // end of namespace
