// Version.cpp

#include "stdafx.h"
#pragma hdrstop

#include "Version.h"
#include "Constants.h"
#include <sstream>

namespace WaveSoftware
{
   
   Version::Version()
   {
      SetMajor(0);
      SetMinor(0);
      SetRelease(0);
      SetBuild(0);

      m_IsValid = true;
   }

   Version::Version(int maj, int min, int rel, int bld)
   {
      SetMajor(maj);
      SetMinor(min);
      SetRelease(rel);
      SetBuild(bld);

      m_IsValid = true;
   }

   Version::Version(const TCHAR* str)
   {
      int maj = 0;
      int min = 0;
      int rel = 0;
      int bld = 0;

      int c = std::_stscanf(str, _T("%d.%d.%d.%d"), &maj, &min, &rel, &bld);

      m_IsValid = false;
   
      if(4 == c)
      {
         SetMajor(maj);
         SetMinor(min);
         SetRelease(rel);
         SetBuild(bld);

         m_IsValid = true;
      }
   }

   int Version::Compare(const Version& v) const
   {
      int dmaj = GetMajor() - v.GetMajor();
      int dmin = GetMinor() - v.GetMinor();
      int drel = GetRelease() - v.GetRelease();
      int dbld = GetBuild() - v.GetBuild();

      if(0 != dmaj) return(dmaj);
      if(0 != dmin) return(dmin);
      if(0 != drel) return(drel);
      if(0 != dbld) return(dbld);

      // Versions are the same.
      return(0);
   }

   std::string Version::ToString() const
   {
      std::stringstream ss;

      ss << m_Major << CSTR_PERIOD << m_Minor << CSTR_PERIOD << m_Release << CSTR_PERIOD << m_Build;

      return(ss.str());
   }

   void Version::SetMajor(int maj)
   {
      m_Major = maj;
   }

   void Version::SetMinor(int min)
   {
      m_Minor = min;
   }

   void Version::SetRelease(int rel)
   {
      m_Release = rel;
   }

   void Version::SetBuild(int bld)
   {
      m_Build = bld;
   }

   int Version::GetMajor() const
   {
      return(m_Major);
   }

   int Version::GetMinor() const
   {
      return(m_Minor);
   }

   int Version::GetRelease() const
   {
      return(m_Release);
   }

   int Version::GetBuild() const
   {
      return(m_Build);
   }

   bool Version::IsValid() const
   {
      return(m_IsValid);
   }

} // end of namespace
