import unittest, os, os.path, shutil, re, string

class ListFailures:
   def __init__(self):
      self.failedTestList = []
   def add(self, fname):
      self.failedTestList += [ fname ]
   def __len__(self):
      return len(self.failedTestList)
   def __str__(self):
      return "\n".join(self.failedTestList)

failures = ListFailures()

############################################################################
# User must specify the compiler call and link options:                    #
#        0= gcc2.9x, 1 = gcc3.0.x, 2 = VC++, 3 = Borland                   #
############################################################################
class CppTestCase(unittest.TestCase):
   def __init__(self, testfun, fname):
      unittest.TestCase.__init__(self, testfun)
      #Compiler and file extensions
      self.compile = [ \
                       "g++ -c -DGCC29x %s.cpp", \
                       "g++ -Wno- -c -DGCC30x %s.cpp", \
                       "cl /w /nologo /c -DMSVC6x %s.cpp", \
                       "bcc32 -w- -q -c -DBORLAND55 %s.cpp",
                       "CC -c -DMIPS %s.cpp" \
                     ]

      self.link = [ \
                       "g++ -o %s.exe %s.o", \
                       "g++ -o %s.exe %s.o", \
                       "cl /nologo /w /Fe%s.exe %s.obj", \
                       "bcc32 -q -e%s.exe %s.obj", \
                       "CC -o %s.exe %s.o" \
                  ]

      #The rest are set automatically:
      self.fileName = fname
      self.toPass = not (fname[:4] == "fail")
      self.hasMain = 0
      self.directory = os.getcwd()

   def setUp(self):
      print "\n********************************************"
      print "\nExecuting: %s.cpp" % self.fileName
      oldFile = open(self.fileName+".cpp", "r")
      currentline = oldFile.readline()
      while currentline:
         if re.search("main", currentline):
            self.hasMain = 1
            break;
         currentline = oldFile.readline()
      oldFile.close()

   def tearDown(self): pass

   # This function determines whether a test case conforms
   # to the ISO standard:
   def testExecute(self):
      # different compilers require different combinations of calls
      executed = 0
      compiled = (os.system(self.compile[0] % self.fileName) == 0)
      if compiled and self.hasMain:
         linked = (os.system(self.link[0] % (self.fileName, self.fileName)) == 0)
         if linked:
            executed = (os.system("%s.exe" % self.fileName) == 0)

      if self.toPass and self.hasMain and compiled and executed:
         print "PASS: Semantics properly supported"

      elif self.toPass and self.hasMain and compiled and not executed:
         print "FAIL: Semantics not supported"
         failures.add(self.fileName)

      elif self.toPass and self.hasMain and not compiled and not executed:
         print "FAIL: Should have compiled"
         failures.add(self.fileName)

      elif self.toPass and not self.hasMain and compiled and not executed:
         print "PASS: Compiled as expected"

      elif self.toPass and not self.hasMain and not compiled and not executed:
         print "FAIL: Should have compiled"
         failures.add(self.fileName)

      elif not self.toPass and self.hasMain and compiled and executed:
         print "FAIL: Executed but shouldn't have"
         failures.add(self.fileName)

      elif not self.toPass and self.hasMain and compiled and not executed:
         print "PASS: Semantics properly supported"

      elif not self.toPass and self.hasMain and not compiled and not executed:
         print "PASS: Did not compile, as expected"

      elif not self.toPass and not self.hasMain and compiled and not executed:
         print "FAIL: Should not have compiled"
         failures.add(self.fileName)

      elif not self.toPass and not self.hasMain and not compiled and not executed:
         print "PASS: Did not compile, as expected"

      else:
         print "logic errors"
